var TwitterSearch;
(function($) {
  TwitterSearch = {
    url: 'http://search.twitter.com/search.json?callback=?',
    cache: null,
    init: function() {
      this.view = $('#twitter-search');
      this.form = this.view.find('form');
      this.result = $('#twitter-search-result');
      this.query = this.form.find(':text');

      var self = this;
      this.form.submit($.proxy(this, 'search', undefined));
      this.result.delegate('.update', 'click', function() {
        if (self.cache) self.search(self.cache.refresh_url.slice(1) +
                                    '&rpp=' + self.form.find('[name=rpp]').val());
      }).delegate('.prev', 'click', function() {
        if (!self.cache) return false;
        if (self.cache.previous_page &&
            self.cache.previous_page.length > 1)
          self.search(self.cache.previous_page.slice(1));
        else if (self.cache.page > 1) {
          self.search($.param({page: self.cache.page - 1,
            max_id: self.cache.max_id}) + '&' + self.form.serialize());
        }
      }).delegate('.next', 'click', function() {
        if (self.cache) self.search(self.cache.next_page.slice(1));
      }).delegate('.search', 'click', function() {
        self.query.val($(this).text());
        self.search();
      });

      this.query.focus();
    },
    search: function(params) {
      if (!params) {
        var trimedQuery = $.trim(this.query.val());
        if (!trimedQuery.length) return false;
        this.query.val(trimedQuery);
        params = this.form.serialize();
      }
      $.getJSON(this.url, params).done($.proxy(this, 'render'));
      return false;
    },
    render: function(data, statusText, jqXHR) {
      this.cache = null;
      this.result.find('.tweet').remove().end()
                 .find('.paginate').empty().end()
                 .find('.not-found').remove();

      if (!data) return false;

      this.cache = data;
      var paginate = this.result.find('.paginate');
      if (data.page && data.page > 1 ||
          data.previous_page && data.previous_page.length > 1) {
        $('<a>', {class: 'prev', href: 'javascript:void(0);', text: '\u00ab 前へ'}).appendTo(paginate);
      }
      if (data.refresh_url && data.refresh_url.length > 1) {
        $('<a>', {class: 'update', href: 'javascript:void(0);', text: '更新'}).appendTo(paginate);
      }
      if (data.next_page && data.next_page.length > 1)  {
        $('<a>', {class: 'next', href: 'javascript:void(0);', text: '次へ \u00bb'}).appendTo(paginate);
      }
      if (data.results && data.results.length > 0) {
        var query = decodeURIComponent(data.query.replace(/\+/g, ' '));
        var list = $('.query-list .search').filter(function() {
          return $(this).text() === query;
        });
        if (!list.length) {
          $('<a>', {class: 'search', href: 'javascript:void(0);', text: query})
            .appendTo(this.result.find('.query-list'));
        }
        $('#twitter-search-tweet-tmpl').tmpl(
          data.results, {query: query, renderText: this.renderText}
        ).appendTo(this.result);
      } else {
        $('<p class="not-found">つぶやきは見つかりませんでした</p>').appendTo(this.result);
      }
    },
    renderText: function(query, text) {
      // "～"に対応
      var wordMatcher = query.replace(/([\/\.\?\,\|\+\-\!\^\$\(\)\{\}\[\]])/g, "\\$1").replace(/((\S+)\s+)(?:OR\s+)?|([^"]+)"?$/g, function(m0, m1, m2, m3) {
        if (m3 && !!m3.length) return m3;
        return /^"/.test(m1) ? m1.replace(/^"/, '') : m2.replace(/"$/, '') + '|';
      });
      var regexQuery = new RegExp('(<[^>]+="[^"]*)?(' + wordMatcher + ')', 'ig');
      return text.replace(
          /(http:\/\/[^"“”「」@\:\(\)\{\}\s]+)/g,
          "<a target=\"_blank\" href=\"$1\">$1</a>")
        .replace(
          /(@\w+|[#＃][^"“”「」@\:\(\)\{\}\s]+)/g,
          "<a class=\"search\" href=\"javascript:void(0);\">$1</a>")
        .replace(regexQuery, function(m0, m1, m2) {
          return (m1 && !!m1.length) ? m0 :
            '<span class="query">' + m2 + '</span>';
        });
    }
  }
})(jQuery);
